package net.lamgc.scext.onedrive_transfer

import org.jetbrains.exposed.sql.Database
import org.jetbrains.exposed.sql.transactions.TransactionManager
import org.jetbrains.exposed.sql.transactions.transaction
import com.microsoft.aad.msal4j.*
import java.sql.Connection
import java.net.URL
import java.net.URI

class OneDriveTransferSettingManager(private val authClient: ConfidentialClientApplication, private val db: Database) {


    init {
        TransactionManager.manager.defaultIsolationLevel = Connection.TRANSACTION_SERIALIZABLE
    }

    fun getTransferSetting(userId: Long): OneDriveTransferSetting? {
        return transaction(db) {
            return@transaction OneDriveTransferSetting.find { OneDriveTransferSettings.telegramUserId eq userId }.firstOrNull()
        }
    }

    fun <R> doSomething(handle: () -> R): R {
        return transaction(db) {
            handle()
        }
    }

    fun createAuthorizationRequest(userId: Long): URL {
        val parameters = AuthorizationRequestUrlParameters
            .builder("http://localhost:45678/", OAUTH2_SCOPE)
            .responseMode(ResponseMode.QUERY)
            .prompt(Prompt.SELECT_ACCOUNT)
            .state(userId.toString())
            .build()
        return authClient.getAuthorizationRequestUrl(parameters)
    }

    fun updateAccount(userId: Long, redirectUrl: URL, checkUserId: Boolean = true): OneDriveTransferSetting {
        val queries = redirectUrl.getQueryMap()
        if (checkUserId && userId != queries["state"]?.toLong()) {
            throw IllegalArgumentException("State 不等于 userId.")
        }
        val future = authClient.acquireToken(
            AuthorizationCodeParameters
                .builder(queries["code"], URI.create("http://localhost:45678/"))
                .build()
        )
        val result = future.get()
        return transaction(db) {
            val account = OneDriveTransferSetting.find { OneDriveTransferSettings.telegramUserId eq userId }.firstOrNull()
            account?.apply {
                accountId = result.account().homeAccountId()
                userName = result.account().username()
            }
                ?: OneDriveTransferSetting.new {
                    val drive = try {
                        OneDriveTransferService.createGraphClient(authClient, result.account())
                            .me().drive()
                            .buildRequest().get()
                    } catch (e: Exception) {
                        null
                    }

                    telegramUserId = userId
                    accountId = result.account().homeAccountId()
                    userName = result.account().username()
                    driveId = drive?.id ?: ""
                    storagePath = "Telegram Files/"
                }
        }
    }

    fun getAccountByTgUserId(userId: Long): OneDriveTransferSetting? {
        return transaction(db) {
            OneDriveTransferSetting.find { OneDriveTransferSettings.telegramUserId eq userId }.firstOrNull()
        }
    }

    companion object {
        const val AUTHORITY = "https://login.microsoftonline.com/common"
        val OAUTH2_SCOPE = setOf(
            "User.Read",
            "Files.Read",
            "Files.ReadWrite",
            "Files.Read.All",
            "Files.ReadWrite.All",
            "Sites.Read.All",
            "Sites.ReadWrite.All",
            "offline_access"
        )
    }
}
